# clustering - AMPL model

# graph
param n >= 1, integer;
set V := 1..n;
set E within {V,V};

# edge weights
param c{E};

# edge inclusions
param I{E};

# vertex colours
param lambda{V};
param gamma{u in V, v in V : u != v} := 
  if (lambda[u] = lambda[v]) then 0 else 1;

# arc colours
param mu{E};

# max number of clusters
param kmax default n;
set K := 1..kmax;

# original problem variables
var x{V,K} binary;

# linearization variables
var w{V,K,V,K} >= 0, <= 1;

# model
minimize intercluster : 
  sum{k in K, l in K, (u,v) in E : k != l} I[u,v] * c[u,v] * w[u,k,v,l];

subject to assignment {v in V} : sum{k in K} x[v,k] = 1;

subject to cardinality {k in K} : sum{v in V} x[v,k] <= ceil(card(V)/kmax)+1;

subject to existence {k in K} : sum{v in V} x[v,k] >= 1;

## to fix some variable values
#subject to fixdb1 : x[11,1] = 1;
#subject to fixdb2 : x[12,1] = 1;
#subject to fixdb3 : x[13,1] = 1;
#subject to fixdb4 : x[14,1] = 1;


subject to redlin {u in V, v in V, k in K : (u,v) in E or (v,u) in E} :
  sum{h in K} w[u,h,v,k] = x[v,k];

subject to symmetry{u in V, v in V, h in K, k in K : (u,v) in E or (v,u) in E}:
  w[u,h,v,k] = w[v,k,u,h];

subject to square{v in V, k in K} : w[v,k,v,k] = x[v,k];

